"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executableOpenAPIMiddlewareQueryParameters = void 0;
const openapi_document_local_ref_resolver_1 = require("openapi-document-local-ref-resolver");
const executable_openapi_parameters_1 = require("executable-openapi-parameters");
const _2020_1 = require("ajv/dist/2020");
const memoize = require("memoizee");
const ts_invariant_1 = require("ts-invariant");
const executableOpenAPIMiddlewareQueryParameters = ({ documentRefResolver, formatErrorResponse = executable_openapi_parameters_1.formatInvalidParameterResponse, formats } = {}) => {
    const ajv = new _2020_1.default({
        coerceTypes: true,
        useDefaults: true,
        removeAdditional: true,
        formats
    });
    const compileAjvBasedValidateCached = memoize(executable_openapi_parameters_1.compileAjvBasedValidate);
    return (handle, parameters, body, context, info) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const originalQueryParameters = (_a = parameters.query) !== null && _a !== void 0 ? _a : {};
        const resolveRef = documentRefResolver !== null && documentRefResolver !== void 0 ? documentRefResolver : (0, openapi_document_local_ref_resolver_1.localRefResolver)(info.document);
        const parametersSpecs = yield (0, executable_openapi_parameters_1.resolveOperationParametersSpecs)(info.operationObject, info.pathItemObject, resolveRef);
        const queryParametersSpecs = parametersSpecs.filter((p) => p.in === 'query');
        const newQueryParameters = {};
        for (const spec of queryParametersSpecs) {
            const { name, schema, required } = spec;
            let value = originalQueryParameters[name];
            if (spec.allowEmptyValue === true && value === '') {
                value = true;
            }
            if (required === true && value === undefined) {
                const error = new executable_openapi_parameters_1.InvalidParameterError(name, 'query', 'is required');
                return yield formatErrorResponse(error, info.operationObject.responses);
            }
            // validates and coerce
            if (schema !== undefined) {
                const validate = compileAjvBasedValidateCached(ajv, schema);
                const { pass, errors, data: newValue } = validate(value);
                if (!pass) {
                    (0, ts_invariant_1.default)(errors, '`validate` should return defined `errors` when `pass` is true');
                    const error = new executable_openapi_parameters_1.InvalidParameterError(name, 'query', errors);
                    return yield formatErrorResponse(error, info.operationObject.responses);
                }
                newQueryParameters[name] = newValue;
            }
            else {
                newQueryParameters[name] = value;
            }
        }
        return yield handle(Object.assign(Object.assign({}, parameters), { query: newQueryParameters }), body, context, info);
    });
};
exports.executableOpenAPIMiddlewareQueryParameters = executableOpenAPIMiddlewareQueryParameters;
//# sourceMappingURL=middleware.js.map